#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from heat.common import exception
from heat.engine.resources.openstack.heat import structured_config as sc
from heat.engine import rsrc_defn
from heat.engine import software_config_io as swc_io
from heat.engine import stack as parser
from heat.engine import template
from heat.rpc import api as rpc_api
from heat.tests import common
from heat.tests import utils


SCENARIOS = [
    (
        'no_functions',
        dict(input_key='get_input',
             inputs={},
             config={'foo': 'bar'},
             result={'foo': 'bar'}),
    ),
    (
        'none_inputs',
        dict(input_key='get_input',
             inputs=None,
             config={'foo': 'bar'},
             result={'foo': 'bar'}),
    ),
    (
        'none_config',
        dict(input_key='get_input',
             inputs=None,
             config=None,
             result=None),
    ),
    (
        'empty_config',
        dict(input_key='get_input',
             inputs=None,
             config='',
             result=''),
    ),
    (
        'simple',
        dict(input_key='get_input',
             inputs={'bar': 'baa'},
             config={'foo': {'get_input': 'bar'}},
             result={'foo': 'baa'}),
    ),
    (
        'multi_key',
        dict(input_key='get_input',
             inputs={'bar': 'baa'},
             config={'foo': [{'get_input': 'bar'}, 'other']},
             result={'foo': ['baa', 'other']}),
    ),
    (
        'list_arg',
        dict(input_key='get_input',
             inputs={'bar': 'baa'},
             config={'foo': {'get_input': ['bar', 'baz']}},
             result={'foo': {'get_input': ['bar', 'baz']}}),
    ),
    (
        'missing_input',
        dict(input_key='get_input',
             inputs={'bar': 'baa'},
             config={'foo': {'get_input': 'barr'}},
             result={'foo': None}),
    ),
    (
        'deep',
        dict(input_key='get_input',
             inputs={'bar': 'baa'},
             config={'foo': {'foo': {'get_input': 'bar'}}},
             result={'foo': {'foo': 'baa'}}),
    ),
    (
        'shallow',
        dict(input_key='get_input',
             inputs={'bar': 'baa'},
             config={'get_input': 'bar'},
             result='baa'),
    ),
    (
        'list',
        dict(input_key='get_input',
             inputs={'bar': 'baa', 'bar2': 'baz', 'bar3': 'bink'},
             config={'foo': [
                 {'get_input': 'bar'},
                 {'get_input': 'bar2'},
                 {'get_input': 'bar3'}]},
             result={'foo': ['baa', 'baz', 'bink']}),
    )
]


class StructuredConfigTestJSON(common.HeatTestCase):

    template = {
        'HeatTemplateFormatVersion': '2012-12-12',
        'Resources': {
            'config_mysql': {
                'Type': 'OS::Heat::StructuredConfig',
                'Properties': {'config': {'foo': 'bar'}}
            }
        }
    }

    stored_config = {'foo': 'bar'}

    def setUp(self):
        super(StructuredConfigTestJSON, self).setUp()
        self.ctx = utils.dummy_context()
        self.properties = {
            'config': {'foo': 'bar'}
        }
        self.stack = parser.Stack(
            self.ctx, 'software_config_test_stack',
            template.Template(self.template))
        self.config = self.stack['config_mysql']
        self.rpc_client = mock.MagicMock()
        self.config._rpc_client = self.rpc_client

    def test_handle_create(self):
        config_id = 'c8a19429-7fde-47ea-a42f-40045488226c'
        value = {'id': config_id}
        self.rpc_client.create_software_config.return_value = value
        self.config.handle_create()
        self.assertEqual(config_id, self.config.resource_id)
        kwargs = self.rpc_client.create_software_config.call_args[1]
        self.assertEqual(self.stored_config, kwargs['config'])


class StructuredDeploymentDerivedTest(common.HeatTestCase):

    template = {
        'HeatTemplateFormatVersion': '2012-12-12',
        'Resources': {
            'deploy_mysql': {
                'Type': 'OS::Heat::StructuredDeployment'
            }
        }
    }

    def setUp(self):
        super(StructuredDeploymentDerivedTest, self).setUp()
        self.ctx = utils.dummy_context()
        props = {
            'server': '9f1f0e00-05d2-4ca5-8602-95021f19c9d0',
            'input_values': {'bar': 'baz'},
        }
        self.template['Resources']['deploy_mysql']['Properties'] = props
        self.stack = parser.Stack(
            self.ctx, 'software_deploly_test_stack',
            template.Template(self.template))
        self.deployment = self.stack['deploy_mysql']

    def test_build_derived_config(self):
        source = {
            'config': {"foo": {"get_input": "bar"}}
        }
        inputs = [swc_io.InputConfig(name='bar', value='baz')]
        result = self.deployment._build_derived_config(
            'CREATE', source, inputs, {})
        self.assertEqual({"foo": "baz"}, result)

    def test_build_derived_config_params_with_empty_config(self):
        source = {}
        source[rpc_api.SOFTWARE_CONFIG_INPUTS] = []
        source[rpc_api.SOFTWARE_CONFIG_OUTPUTS] = []
        result = self.deployment._build_derived_config_params(
            'CREATE', source)
        self.assertEqual('Heat::Ungrouped', result['group'])
        self.assertEqual({}, result['config'])
        self.assertEqual(self.deployment.physical_resource_name(),
                         result['name'])
        self.assertIn({'name': 'bar', 'type': 'String', 'value': 'baz'},
                      result['inputs'])
        self.assertIsNone(result['options'])
        self.assertEqual([], result['outputs'])


class StructuredDeploymentWithStrictInputTest(common.HeatTestCase):

    template = {
        'HeatTemplateFormatVersion': '2012-12-12',
        'Resources': {
            'deploy_mysql': {
                'Type': 'OS::Heat::StructuredDeployment',
                'Properties': {}
            }
        }
    }

    def setUp(self):
        super(StructuredDeploymentWithStrictInputTest, self).setUp()
        self.source = {'config':
                       {'foo': [{"get_input": "bar"},
                                {"get_input": "barz"}]}}
        self.inputs = [swc_io.InputConfig(name='bar', value='baz'),
                       swc_io.InputConfig(name='barz', value='baz2')]

    def _stack_with_template(self, template_def):
        self.ctx = utils.dummy_context()
        self.stack = parser.Stack(
            self.ctx, 'software_deploly_test_stack',
            template.Template(template_def))
        self.deployment = self.stack['deploy_mysql']

    def test_build_derived_config_failure(self):
        props = {'input_values_validate': 'STRICT'}
        self.template['Resources']['deploy_mysql']['Properties'] = props
        self._stack_with_template(self.template)

        self.assertRaises(exception.UserParameterMissing,
                          self.deployment._build_derived_config,
                          'CREATE', self.source, self.inputs[:1], {})

    def test_build_derived_config_success(self):
        props = {'input_values_validate': 'STRICT'}
        self.template['Resources']['deploy_mysql']['Properties'] = props
        self._stack_with_template(self.template)

        expected = {'foo': ['baz', 'baz2']}
        result = self.deployment._build_derived_config(
            'CREATE', self.source, self.inputs, {})
        self.assertEqual(expected, result)


class StructuredDeploymentParseTest(common.HeatTestCase):
    scenarios = SCENARIOS

    def test_parse(self):
        parse = sc.StructuredDeployment.parse
        self.assertEqual(
            self.result,
            parse(self.inputs, self.input_key, self.config))


class StructuredDeploymentGroupTest(common.HeatTestCase):

    template = {
        'heat_template_version': '2013-05-23',
        'resources': {
            'deploy_mysql': {
                'type': 'OS::Heat::StructuredDeploymentGroup',
                'properties': {
                    'config': 'config_uuid',
                    'servers': {'server1': 'uuid1', 'server2': 'uuid2'},
                }
            }
        }
    }

    def test_build_resource_definition(self):
        stack = utils.parse_stack(self.template)
        snip = stack.t.resource_definitions(stack)['deploy_mysql']
        resg = sc.StructuredDeploymentGroup('test', snip, stack)
        expect = rsrc_defn.ResourceDefinition(
            None,
            'OS::Heat::StructuredDeployment',
            {'actions': ['CREATE', 'UPDATE'],
             'config': 'config_uuid',
             'input_values': None,
             'name': None,
             'server': 'uuid1',
             'input_key': 'get_input',
             'signal_transport': 'CFN_SIGNAL',
             'input_values_validate': 'LAX'})

        rdef = resg.get_resource_def()
        self.assertEqual(
            expect, resg.build_resource_definition('server1', rdef))
        rdef = resg.get_resource_def(include_all=True)
        self.assertEqual(
            expect, resg.build_resource_definition('server1', rdef))

    def test_resource_names(self):
        stack = utils.parse_stack(self.template)
        snip = stack.t.resource_definitions(stack)['deploy_mysql']
        resg = sc.StructuredDeploymentGroup('test', snip, stack)
        self.assertEqual(
            set(('server1', 'server2')),
            set(resg._resource_names())
        )

        resg.properties = {'servers': {'s1': 'u1', 's2': 'u2', 's3': 'u3'}}
        self.assertEqual(
            set(('s1', 's2', 's3')),
            set(resg._resource_names()))

    def test_assemble_nested(self):
        """Tests nested stack implements group creation based on properties.

        Tests that the nested stack that implements the group is created
        appropriately based on properties.
        """
        stack = utils.parse_stack(self.template)
        snip = stack.t.resource_definitions(stack)['deploy_mysql']
        resg = sc.StructuredDeploymentGroup('test', snip, stack)
        templ = {
            "heat_template_version": "2015-04-30",
            "resources": {
                "server1": {
                    'type': 'OS::Heat::StructuredDeployment',
                    'properties': {
                        'server': 'uuid1',
                        'actions': ['CREATE', 'UPDATE'],
                        'config': 'config_uuid',
                        'input_key': 'get_input',
                        'input_values': None,
                        'name': None,
                        'signal_transport': 'CFN_SIGNAL',
                        'input_values_validate': 'LAX'
                    }
                },
                "server2": {
                    'type': 'OS::Heat::StructuredDeployment',
                    'properties': {
                        'server': 'uuid2',
                        'actions': ['CREATE', 'UPDATE'],
                        'config': 'config_uuid',
                        'input_key': 'get_input',
                        'input_values': None,
                        'name': None,
                        'signal_transport': 'CFN_SIGNAL',
                        'input_values_validate': 'LAX'
                    }
                }
            }
        }

        self.assertEqual(templ, resg._assemble_nested(['server1',
                                                       'server2']).t)


class StructuredDeploymentWithStrictInputParseTest(common.HeatTestCase):
    scenarios = SCENARIOS

    def test_parse(self):
        self.parse = sc.StructuredDeployment.parse
        if 'missing_input' not in self.shortDescription():
            self.assertEqual(
                self.result,
                self.parse(
                    self.inputs,
                    self.input_key,
                    self.config,
                    check_input_val='STRICT')
            )
        else:
            self.assertRaises(exception.UserParameterMissing,
                              self.parse,
                              self.inputs,
                              self.input_key,
                              self.config,
                              check_input_val='STRICT')


class StructuredDeploymentParseMethodsTest(common.HeatTestCase):
    def test_get_key_args(self):
        snippet = {'get_input': 'bar'}
        input_key = 'get_input'
        expected = 'bar'
        result = sc.StructuredDeployment.get_input_key_arg(snippet, input_key)
        self.assertEqual(expected, result)

    def test_get_key_args_long_snippet(self):
        snippet = {'get_input': 'bar', 'second': 'foo'}
        input_key = 'get_input'
        result = sc.StructuredDeployment.get_input_key_arg(snippet, input_key)
        self.assertFalse(result)

    def test_get_key_args_unknown_input_key(self):
        snippet = {'get_input': 'bar'}
        input_key = 'input'
        result = sc.StructuredDeployment.get_input_key_arg(snippet, input_key)
        self.assertFalse(result)

    def test_get_key_args_wrong_args(self):
        snippet = {'get_input': None}
        input_key = 'get_input'
        result = sc.StructuredDeployment.get_input_key_arg(snippet, input_key)
        self.assertFalse(result)

    def test_get_input_key_value(self):
        inputs = {'bar': 'baz', 'foo': 'foo2'}
        res = sc.StructuredDeployment.get_input_key_value('bar', inputs, False)
        expected = 'baz'
        self.assertEqual(expected, res)

    def test_get_input_key_value_raise_exception(self):
        inputs = {'bar': 'baz', 'foo': 'foo2'}
        self.assertRaises(exception.UserParameterMissing,
                          sc.StructuredDeployment.get_input_key_value,
                          'barz',
                          inputs,
                          'STRICT')

    def test_get_input_key_value_get_none(self):
        inputs = {'bar': 'baz', 'foo': 'foo2'}
        res = sc.StructuredDeployment.get_input_key_value('brz', inputs, False)
        self.assertIsNone(res)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# SQLAlchemy helper functions

from sqlalchemy.orm import exc
import tenacity


def retry_on_stale_data_error(func):
    wrapper = tenacity.retry(
        stop=tenacity.stop_after_attempt(3),
        retry=tenacity.retry_if_exception_type(exc.StaleDataError),
        reraise=True)
    return wrapper(func)



# Copyright 2018 Ericsson
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import collections

from heat.common.i18n import _
from heat.engine import properties
from heat.engine.resources.openstack.neutron import neutron
from heat.engine import support


class L2Gateway(neutron.NeutronResource):
    """A resource for managing Neutron L2 Gateways.

    The are a number of use cases that can be addressed by an L2 Gateway API.
    Most notably in cloud computing environments, a typical use case is
    bridging the virtual with the physical. Translate this to Neutron and the
    OpenStack world, and this means relying on L2 Gateway capabilities to
    extend Neutron logical (overlay) networks into physical (provider)
    networks that are outside the OpenStack realm.
    """

    required_service_extension = 'l2-gateway'

    entity = 'l2_gateway'

    support_status = support.SupportStatus(version='12.0.0')

    PROPERTIES = (
        NAME, DEVICES,
    ) = (
        'name', 'devices',
    )

    _DEVICE_KEYS = (
        DEVICE_NAME, INTERFACES,
    ) = (
        'device_name', 'interfaces',
    )

    _INTERFACE_KEYS = (
        INTERFACE_NAME, SEGMENTATION_ID,
    ) = (
        'name', 'segmentation_id',
    )

    _interface_schema = {
        INTERFACE_NAME: properties.Schema(
            properties.Schema.STRING,
            _('The name of the interface on the gateway device.'),
            required=True
        ),
        SEGMENTATION_ID: properties.Schema(
            properties.Schema.LIST,
            _('A list of segmentation ids of the interface.')
        ),
    }

    _device_schema = {
        DEVICE_NAME: properties.Schema(
            properties.Schema.STRING,
            _('The name of the gateway device.'),
            required=True
        ),
        INTERFACES: properties.Schema(
            properties.Schema.LIST,
            _('List of gateway device interfaces.'),
            schema=properties.Schema(
                properties.Schema.MAP,
                schema=_interface_schema
            ),
            required=True
        )
    }

    properties_schema = {
        NAME: properties.Schema(
            properties.Schema.STRING,
            _('A symbolic name for the l2-gateway, '
              'which is not required to be unique.'),
            required=True,
            update_allowed=True
        ),
        DEVICES: properties.Schema(
            properties.Schema.LIST,
            _('List of gateway devices.'),
            schema=properties.Schema(
                properties.Schema.MAP,
                schema=_device_schema
            ),
            required=True,
            update_allowed=True
        ),
    }

    @staticmethod
    def _remove_none_value_props(props):
        if isinstance(props, collections.abc.Mapping):
            return dict((k, L2Gateway._remove_none_value_props(v)) for k, v
                        in props.items() if v is not None)
        elif (isinstance(props, collections.abc.Sequence) and
              not isinstance(props, str)):
            return list(L2Gateway._remove_none_value_props(p) for p in props
                        if p is not None)
        return props

    @staticmethod
    def prepare_properties(properties, name):
        # Overrides method from base class NeutronResource to ensure None
        # values are removed from all levels of value_specs.

        # TODO(neatherweb): move this recursive check for None to
        # prepare_properties in NeutronResource
        props = L2Gateway._remove_none_value_props(dict(properties))

        if 'name' in properties:
            props.setdefault('name', name)

        return props

    def handle_create(self):
        props = self.prepare_properties(
            self.properties,
            self.physical_resource_name())
        l2gw = self.client().create_l2_gateway(
            {'l2_gateway': props})['l2_gateway']
        self.resource_id_set(l2gw['id'])

    def handle_delete(self):
        if self.resource_id is None:
            return
        with self.client_plugin().ignore_not_found:
            self.client().delete_l2_gateway(self.resource_id)

    def handle_update(self, json_snippet, tmpl_diff, prop_diff):
        if prop_diff:
            self.prepare_update_properties(prop_diff)
            prop_diff = L2Gateway._remove_none_value_props(prop_diff)
            self.client().update_l2_gateway(
                self.resource_id, {'l2_gateway': prop_diff})


def resource_mapping():
    return {
        'OS::Neutron::L2Gateway': L2Gateway,
    }



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from heat.tests import common
from heat.tests import utils


class BlazarClientPluginTest(common.HeatTestCase):

    def setUp(self):
        super(BlazarClientPluginTest, self).setUp()
        self.blazar_client = mock.MagicMock()
        context = utils.dummy_context()
        self.blazar_client_plugin = context.clients.client_plugin('blazar')

    def _stub_client(self):
        self.blazar_client_plugin.client = lambda: self.blazar_client

    def test_create(self):
        client = self.blazar_client_plugin.client()
        self.assertEqual(None, client.blazar_url)

    def test_has_host_pass(self):
        self._stub_client()
        self.blazar_client.host.list.return_value = ['hosta']
        self.assertEqual(True, self.blazar_client_plugin.has_host())

    def test_has_host_fail(self):
        self._stub_client()
        self.blazar_client.host.list.return_value = []
        self.assertEqual(False, self.blazar_client_plugin.has_host())



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import uuid

from oslo_utils import timeutils

from heat.rpc import listener_client

SERVICE_KEYS = (
    SERVICE_ID,
    SERVICE_HOST,
    SERVICE_HOSTNAME,
    SERVICE_BINARY,
    SERVICE_TOPIC,
    SERVICE_ENGINE_ID,
    SERVICE_REPORT_INTERVAL,
    SERVICE_CREATED_AT,
    SERVICE_UPDATED_AT,
    SERVICE_DELETED_AT,
    SERVICE_STATUS
) = (
    'id',
    'host',
    'hostname',
    'binary',
    'topic',
    'engine_id',
    'report_interval',
    'created_at',
    'updated_at',
    'deleted_at',
    'status'
)


def format_service(service):
    if service is None:
        return

    status = 'down'
    last_updated = service.updated_at or service.created_at
    check_interval = (timeutils.utcnow() - last_updated).total_seconds()
    if check_interval <= 2 * service.report_interval:
        status = 'up'

    result = {
        SERVICE_ID: service.id,
        SERVICE_BINARY: service.binary,
        SERVICE_ENGINE_ID: service.engine_id,
        SERVICE_HOST: service.host,
        SERVICE_HOSTNAME: service.hostname,
        SERVICE_TOPIC: service.topic,
        SERVICE_REPORT_INTERVAL: service.report_interval,
        SERVICE_CREATED_AT: service.created_at,
        SERVICE_UPDATED_AT: service.updated_at,
        SERVICE_DELETED_AT: service.deleted_at,
        SERVICE_STATUS: status
    }
    return result


def engine_alive(context, engine_id):
    return listener_client.EngineListenerClient(
        engine_id).is_alive(context)


def generate_engine_id():
    return str(uuid.uuid4())



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from barbicanclient import exceptions
from barbicanclient.v1 import client as barbican_client
from barbicanclient.v1 import containers
from oslo_config import cfg
from oslo_log import log as logging

from heat.common import exception
from heat.engine.clients import client_plugin
from heat.engine import constraints

LOG = logging.getLogger(__name__)

CLIENT_NAME = 'barbican'


class BarbicanClientPlugin(client_plugin.ClientPlugin):

    service_types = [KEY_MANAGER] = ['key-manager']

    def _create(self):
        interface = self._get_client_option(CLIENT_NAME, 'endpoint_type')
        client = barbican_client.Client(
            session=self.context.keystone_session,
            service_type=self.KEY_MANAGER,
            interface=interface,
            connect_retries=cfg.CONF.client_retry_limit,
            region_name=self._get_region_name())
        return client

    def is_not_found(self, ex):
        return (isinstance(ex, exceptions.HTTPClientError) and
                ex.status_code == 404)

    def create_generic_container(self, **props):
        return containers.Container(
            self.client().containers._api, **props)

    def create_certificate(self, **props):
        return containers.CertificateContainer(
            self.client().containers._api, **props)

    def create_rsa(self, **props):
        return containers.RSAContainer(
            self.client().containers._api, **props)

    def get_secret_by_ref(self, secret_ref):
        try:
            secret = self.client().secrets.get(secret_ref)
            # Force lazy loading. TODO(therve): replace with to_dict()
            secret.name
            return secret
        except Exception as ex:
            if self.is_not_found(ex):
                raise exception.EntityNotFound(
                    entity="Secret",
                    name=secret_ref)
            LOG.info('Failed to get Barbican secret from reference %s' % (
                secret_ref))
            raise

    def get_secret_payload_by_ref(self, secret_ref):
        return self.get_secret_by_ref(secret_ref).payload

    def get_container_by_ref(self, container_ref):
        try:
            # TODO(therve): replace with to_dict()
            return self.client().containers.get(container_ref)
        except Exception as ex:
            if self.is_not_found(ex):
                raise exception.EntityNotFound(
                    entity="Container",
                    name=container_ref)
            raise


class SecretConstraint(constraints.BaseCustomConstraint):
    resource_client_name = CLIENT_NAME
    resource_getter_name = 'get_secret_by_ref'
    expected_exceptions = (exception.EntityNotFound,)


class ContainerConstraint(constraints.BaseCustomConstraint):
    resource_client_name = CLIENT_NAME
    resource_getter_name = 'get_container_by_ref'
    expected_exceptions = (exception.EntityNotFound,)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from aodhclient import client as ac
from aodhclient import exceptions
from oslo_config import cfg

from heat.engine.clients import client_plugin

CLIENT_NAME = 'aodh'


class AodhClientPlugin(client_plugin.ClientPlugin):

    exceptions_module = exceptions

    service_types = [ALARMING] = ['alarming']

    supported_versions = [V2] = ['2']

    default_version = V2

    def _create(self, version=None):
        interface = self._get_client_option(CLIENT_NAME, 'endpoint_type')

        return ac.Client(
            version,
            session=self.context.keystone_session,
            interface=interface,
            service_type=self.ALARMING,
            connect_retries=cfg.CONF.client_retry_limit,
            region_name=self._get_region_name())

    def is_not_found(self, ex):
        return isinstance(ex, exceptions.NotFound)

    def is_over_limit(self, ex):
        return isinstance(ex, exceptions.OverLimit)

    def is_conflict(self, ex):
        return isinstance(ex, exceptions.Conflict)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from heat.common import exception
from heat.common.i18n import _
from heat.engine import properties
from heat.engine import resource


class AccessPolicy(resource.Resource):
    """Resource for defining which resources can be accessed by users.

    NOTE: Now this resource is actually associated with an AWS user resource,
    not any OS:: resource though it is registered under the OS namespace below.

    Resource for defining resources that users are allowed to access by the
    DescribeStackResource API.
    """
    PROPERTIES = (
        ALLOWED_RESOURCES,
    ) = (
        'AllowedResources',
    )

    properties_schema = {
        ALLOWED_RESOURCES: properties.Schema(
            properties.Schema.LIST,
            _('Resources that users are allowed to access by the '
              'DescribeStackResource API.'),
            required=True
        ),
    }

    def handle_create(self):
        pass

    def validate(self):
        """Make sure all the AllowedResources are present."""
        super(AccessPolicy, self).validate()

        resources = self.properties[self.ALLOWED_RESOURCES]
        # All of the provided resource names must exist in this stack
        for res in resources:
            if res not in self.stack:
                msg = _("AccessPolicy resource %s not in stack") % res
                raise exception.StackValidationFailed(message=msg)

    def access_allowed(self, resource_name):
        return resource_name in self.properties[self.ALLOWED_RESOURCES]


def resource_mapping():
    return {
        'OS::Heat::AccessPolicy': AccessPolicy,
    }



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging

from heat.common import exception
from heat.common.i18n import _
from heat.engine import attributes
from heat.engine import constraints
from heat.engine import properties
from heat.engine import resource
from heat.engine import support

LOG = logging.getLogger(__name__)


class NovaFloatingIp(resource.Resource):
    """A resource for managing Nova floating IPs.

    Floating IP addresses can change their association between instances by
    action of the user. One of the most common use cases for floating IPs is
    to provide public IP addresses to a private cloud, where there are a
    limited number of IP addresses available. Another is for a public cloud
    user to have a "static" IP address that can be reassigned when an instance
    is upgraded or moved.
    """

    deprecation_msg = _('Please use OS::Neutron::FloatingIP instead.')
    support_status = support.SupportStatus(
        status=support.HIDDEN,
        message=deprecation_msg,
        version='11.0.0',
        previous_status=support.SupportStatus(
            status=support.DEPRECATED,
            message=deprecation_msg,
            version='9.0.0',
            previous_status=support.SupportStatus(version='2014.1')
        )
    )

    PROPERTIES = (POOL,) = ('pool',)

    ATTRIBUTES = (
        POOL_ATTR, IP,
    ) = (
        'pool', 'ip',
    )

    properties_schema = {
        POOL: properties.Schema(
            properties.Schema.STRING,
            description=_('Allocate a floating IP from a given '
                          'floating IP pool. Now that nova-network '
                          'is not supported this represents the '
                          'external network.')
        ),
    }

    attributes_schema = {
        POOL_ATTR: attributes.Schema(
            _('Pool from which floating IP is allocated.'),
            type=attributes.Schema.STRING
        ),
        IP: attributes.Schema(
            _('Allocated floating IP address.'),
            type=attributes.Schema.STRING
        ),
    }

    def __init__(self, name, json_snippet, stack):
        super(NovaFloatingIp, self).__init__(name, json_snippet, stack)
        self._floating_ip = None

    def _get_resource(self):
        if self._floating_ip is None and self.resource_id is not None:
            self._floating_ip = self.neutron().show_floatingip(
                self.resource_id)

        return self._floating_ip

    def get_external_network_id(self, pool=None):
        if pool:
            neutron_plugin = self.client_plugin('neutron')
            return neutron_plugin.find_resourceid_by_name_or_id(
                neutron_plugin.RES_TYPE_NETWORK,
                pool)
        ext_filter = {'router:external': True}
        ext_nets = self.neutron().list_networks(**ext_filter)['networks']
        if len(ext_nets) != 1:
            raise exception.Error(
                _('Expected 1 external network, found %d') % len(ext_nets))
        external_network_id = ext_nets[0]['id']
        return external_network_id

    def handle_create(self):
        ext_net_id = self.get_external_network_id(
            pool=self.properties[self.POOL])
        floating_ip = self.neutron().create_floatingip(
            {'floatingip': {'floating_network_id': ext_net_id}})
        self.resource_id_set(floating_ip['floatingip']['id'])
        self._floating_ip = floating_ip

    def handle_delete(self):
        with self.client_plugin('neutron').ignore_not_found:
            self.neutron().delete_floatingip(self.resource_id)

    def _resolve_attribute(self, key):
        if self.resource_id is None:
            return
        floating_ip = self._get_resource()
        attributes = {
            self.POOL_ATTR: floating_ip['floatingip']['floating_network_id'],
            self.IP: floating_ip['floatingip']['floating_ip_address']
        }
        return str(attributes[key])


class NovaFloatingIpAssociation(resource.Resource):
    """A resource associates Nova floating IP with Nova server resource.

    Resource for associating existing Nova floating IP and Nova server.
    """

    deprecation_msg = _(
        'Please use OS::Neutron::FloatingIPAssociation instead.')
    support_status = support.SupportStatus(
        status=support.HIDDEN,
        message=deprecation_msg,
        version='11.0.0',
        previous_status=support.SupportStatus(
            status=support.DEPRECATED,
            message=deprecation_msg,
            version='9.0.0',
            previous_status=support.SupportStatus(version='2014.1')
        )
    )

    PROPERTIES = (
        SERVER, FLOATING_IP
    ) = (
        'server_id', 'floating_ip'
    )

    properties_schema = {
        SERVER: properties.Schema(
            properties.Schema.STRING,
            _('Server to assign floating IP to.'),
            required=True,
            update_allowed=True,
            constraints=[
                constraints.CustomConstraint('nova.server')
            ]
        ),
        FLOATING_IP: properties.Schema(
            properties.Schema.STRING,
            _('ID of the floating IP to assign to the server.'),
            required=True,
            update_allowed=True
        ),
    }

    default_client_name = 'nova'

    def get_reference_id(self):
        return self.physical_resource_name_or_FnGetRefId()

    def handle_create(self):
        self.client_plugin().associate_floatingip(
            self.properties[self.SERVER], self.properties[self.FLOATING_IP])
        self.resource_id_set(self.id)

    def handle_delete(self):
        if self.resource_id is None:
            return
        with self.client_plugin().ignore_not_found:
            self.client_plugin().dissociate_floatingip(
                self.properties[self.FLOATING_IP])

    def handle_update(self, json_snippet, tmpl_diff, prop_diff):
        if prop_diff:
            # If floating_ip in prop_diff, we need to remove the old floating
            # ip from the old server, and then to add the new floating ip
            # to the old/new(if the server_id is changed) server.
            if self.FLOATING_IP in prop_diff:
                self.handle_delete()
            server_id = (prop_diff.get(self.SERVER) or
                         self.properties[self.SERVER])
            fl_ip_id = (prop_diff.get(self.FLOATING_IP) or
                        self.properties[self.FLOATING_IP])
            self.client_plugin().associate_floatingip(server_id, fl_ip_id)
            self.resource_id_set(self.id)


def resource_mapping():
    return {
        'OS::Nova::FloatingIP': NovaFloatingIp,
        'OS::Nova::FloatingIPAssociation': NovaFloatingIpAssociation,
    }



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from oslo_serialization import jsonutils
from oslo_utils import uuidutils
from urllib import parse as urlparse

from heat.common import exception
from heat.common import template_format
from heat.engine.clients.os import heat_plugin
from heat.engine.clients.os import swift
from heat.engine.clients.os import zaqar
from heat.engine import environment
from heat.engine.resources.openstack.heat import deployed_server
from heat.engine import scheduler
from heat.engine import stack as parser
from heat.engine import template
from heat.tests import common
from heat.tests import utils

ds_tmpl = """
heat_template_version: 2015-10-15
resources:
  server:
    type: OS::Heat::DeployedServer
    properties:
      software_config_transport: POLL_TEMP_URL
"""

server_sc_tmpl = """
heat_template_version: 2015-10-15
resources:
  server:
    type: OS::Heat::DeployedServer
    properties:
      software_config_transport: POLL_SERVER_CFN
"""

server_heat_tmpl = """
heat_template_version: 2015-10-15
resources:
  server:
    type: OS::Heat::DeployedServer
    properties:
      software_config_transport: POLL_SERVER_HEAT
"""

server_zaqar_tmpl = """
heat_template_version: 2015-10-15
resources:
  server:
    type: OS::Heat::DeployedServer
    properties:
      software_config_transport: ZAQAR_MESSAGE
"""

ds_deployment_data_tmpl = """
heat_template_version: 2015-10-15
resources:
  server:
    type: OS::Heat::DeployedServer
    properties:
      software_config_transport: POLL_TEMP_URL
      deployment_swift_data:
        container: my-custom-container
        object: my-custom-object
"""

ds_deployment_data_bad_container_tmpl = """
heat_template_version: 2015-10-15
resources:
  server:
    type: OS::Heat::DeployedServer
    properties:
      software_config_transport: POLL_TEMP_URL
      deployment_swift_data:
        container: ''
        object: 'my-custom-object'
"""

ds_deployment_data_bad_object_tmpl = """
heat_template_version: 2015-10-15
resources:
  server:
    type: OS::Heat::DeployedServer
    properties:
      software_config_transport: POLL_TEMP_URL
      deployment_swift_data:
        container: 'my-custom-container'
        object: ''
"""

ds_deployment_data_none_container_tmpl = """
heat_template_version: 2015-10-15
resources:
  server:
    type: OS::Heat::DeployedServer
    properties:
      software_config_transport: POLL_TEMP_URL
      deployment_swift_data:
        container: 0
        object: 'my-custom-object'
"""

ds_deployment_data_none_object_tmpl = """
heat_template_version: 2015-10-15
resources:
  server:
    type: OS::Heat::DeployedServer
    properties:
      software_config_transport: POLL_TEMP_URL
      deployment_swift_data:
        container: 'my-custom-container'
        object: 0
"""


class DeployedServersTest(common.HeatTestCase):
    def _create_test_server(self, name, override_name=False):
        server = self._setup_test_server(name, override_name)
        scheduler.TaskRunner(server.create)()
        return server

    def _setup_test_stack(self, stack_name, test_templ=ds_tmpl):
        t = template_format.parse(test_templ)
        tmpl = template.Template(t, env=environment.Environment())
        stack = parser.Stack(utils.dummy_context(region_name="RegionOne"),
                             stack_name, tmpl,
                             stack_id=uuidutils.generate_uuid(),
                             stack_user_project_id='8888')
        return (tmpl, stack)

    def _server_create_software_config_poll_temp_url(self,
                                                     server_name='server'):
        stack_name = '%s_s' % server_name
        (tmpl, stack) = self._setup_test_stack(stack_name)

        props = tmpl.t['resources']['server']['properties']
        props['software_config_transport'] = 'POLL_TEMP_URL'
        self.server_props = props

        resource_defns = tmpl.resource_definitions(stack)
        server = deployed_server.DeployedServer(
            server_name, resource_defns[server_name], stack)

        sc = mock.Mock()
        sc.head_account.return_value = {
            'x-account-meta-temp-url-key': 'secrit'
        }
        sc.url = 'http://192.0.2.2'

        self.patchobject(swift.SwiftClientPlugin, '_create',
                         return_value=sc)
        scheduler.TaskRunner(server.create)()
        # self._create_test_server(server_name)
        metadata_put_url = server.data().get('metadata_put_url')
        md = server.metadata_get()
        metadata_url = md['os-collect-config']['request']['metadata_url']
        self.assertNotEqual(metadata_url, metadata_put_url)

        container_name = server.physical_resource_name()
        object_name = server.data().get('metadata_object_name')
        self.assertTrue(uuidutils.is_uuid_like(object_name))
        test_path = '/v1/AUTH_test_tenant_id/%s/%s' % (
            server.physical_resource_name(), object_name)
        self.assertEqual(test_path, urlparse.urlparse(metadata_put_url).path)
        self.assertEqual(test_path, urlparse.urlparse(metadata_url).path)
        sc.put_object.assert_called_once_with(
            container_name, object_name, jsonutils.dumps(md))

        sc.head_container.return_value = {'x-container-object-count': '0'}
        server._delete_temp_url()
        sc.delete_object.assert_called_once_with(container_name, object_name)
        sc.head_container.assert_called_once_with(container_name)
        sc.delete_container.assert_called_once_with(container_name)
        return metadata_url, server

    def test_server_create_deployment_swift_data(self):
        server_name = 'server'
        stack_name = '%s_s' % server_name
        (tmpl, stack) = self._setup_test_stack(
            stack_name,
            ds_deployment_data_tmpl)

        props = tmpl.t['resources']['server']['properties']
        props['software_config_transport'] = 'POLL_TEMP_URL'
        self.server_props = props

        resource_defns = tmpl.resource_definitions(stack)
        server = deployed_server.DeployedServer(
            server_name, resource_defns[server_name], stack)

        sc = mock.Mock()
        sc.head_account.return_value = {
            'x-account-meta-temp-url-key': 'secrit'
        }
        sc.url = 'http://192.0.2.2'

        self.patchobject(swift.SwiftClientPlugin, '_create',
                         return_value=sc)
        scheduler.TaskRunner(server.create)()
        # self._create_test_server(server_name)
        metadata_put_url = server.data().get('metadata_put_url')
        md = server.metadata_get()
        metadata_url = md['os-collect-config']['request']['metadata_url']
        self.assertNotEqual(metadata_url, metadata_put_url)

        container_name = 'my-custom-container'
        object_name = 'my-custom-object'
        test_path = '/v1/AUTH_test_tenant_id/%s/%s' % (
            container_name, object_name)
        self.assertEqual(test_path, urlparse.urlparse(metadata_put_url).path)
        self.assertEqual(test_path, urlparse.urlparse(metadata_url).path)
        sc.put_object.assert_called_once_with(
            container_name, object_name, jsonutils.dumps(md))

        sc.head_container.return_value = {'x-container-object-count': '0'}
        server._delete_temp_url()
        sc.delete_object.assert_called_once_with(container_name, object_name)
        sc.head_container.assert_called_once_with(container_name)
        sc.delete_container.assert_called_once_with(container_name)
        return metadata_url, server

    def test_server_create_deployment_swift_data_bad_container(self):
        server_name = 'server'
        stack_name = '%s_s' % server_name
        (tmpl, stack) = self._setup_test_stack(
            stack_name,
            ds_deployment_data_bad_container_tmpl)

        props = tmpl.t['resources']['server']['properties']
        props['software_config_transport'] = 'POLL_TEMP_URL'
        self.server_props = props

        resource_defns = tmpl.resource_definitions(stack)
        server = deployed_server.DeployedServer(
            server_name, resource_defns[server_name], stack)

        self.assertRaises(exception.StackValidationFailed, server.validate)

    def test_server_create_deployment_swift_data_bad_object(self):
        server_name = 'server'
        stack_name = '%s_s' % server_name
        (tmpl, stack) = self._setup_test_stack(
            stack_name,
            ds_deployment_data_bad_object_tmpl)

        props = tmpl.t['resources']['server']['properties']
        props['software_config_transport'] = 'POLL_TEMP_URL'
        self.server_props = props

        resource_defns = tmpl.resource_definitions(stack)
        server = deployed_server.DeployedServer(
            server_name, resource_defns[server_name], stack)

        self.assertRaises(exception.StackValidationFailed, server.validate)

    def test_server_create_deployment_swift_data_none_container(self):
        server_name = 'server'
        stack_name = '%s_s' % server_name
        (tmpl, stack) = self._setup_test_stack(
            stack_name,
            ds_deployment_data_none_container_tmpl)

        props = tmpl.t['resources']['server']['properties']
        props['software_config_transport'] = 'POLL_TEMP_URL'
        self.server_props = props

        resource_defns = tmpl.resource_definitions(stack)
        server = deployed_server.DeployedServer(
            server_name, resource_defns[server_name], stack)

        sc = mock.Mock()
        sc.head_account.return_value = {
            'x-account-meta-temp-url-key': 'secrit'
        }
        sc.url = 'http://192.0.2.2'

        self.patchobject(swift.SwiftClientPlugin, '_create',
                         return_value=sc)
        scheduler.TaskRunner(server.create)()
        # self._create_test_server(server_name)
        metadata_put_url = server.data().get('metadata_put_url')
        md = server.metadata_get()
        metadata_url = md['os-collect-config']['request']['metadata_url']
        self.assertNotEqual(metadata_url, metadata_put_url)

        container_name = '0'
        object_name = 'my-custom-object'
        test_path = '/v1/AUTH_test_tenant_id/%s/%s' % (
            container_name, object_name)
        self.assertEqual(test_path, urlparse.urlparse(metadata_put_url).path)
        self.assertEqual(test_path, urlparse.urlparse(metadata_url).path)
        sc.put_object.assert_called_once_with(
            container_name, object_name, jsonutils.dumps(md))

        sc.head_container.return_value = {'x-container-object-count': '0'}
        server._delete_temp_url()
        sc.delete_object.assert_called_once_with(container_name, object_name)
        sc.head_container.assert_called_once_with(container_name)
        sc.delete_container.assert_called_once_with(container_name)
        return metadata_url, server

    def test_server_create_deployment_swift_data_none_object(self):
        server_name = 'server'
        stack_name = '%s_s' % server_name
        (tmpl, stack) = self._setup_test_stack(
            stack_name,
            ds_deployment_data_none_object_tmpl)

        props = tmpl.t['resources']['server']['properties']
        props['software_config_transport'] = 'POLL_TEMP_URL'
        self.server_props = props

        resource_defns = tmpl.resource_definitions(stack)
        server = deployed_server.DeployedServer(
            server_name, resource_defns[server_name], stack)

        sc = mock.Mock()
        sc.head_account.return_value = {
            'x-account-meta-temp-url-key': 'secrit'
        }
        sc.url = 'http://192.0.2.2'

        self.patchobject(swift.SwiftClientPlugin, '_create',
                         return_value=sc)
        scheduler.TaskRunner(server.create)()
        # self._create_test_server(server_name)
        metadata_put_url = server.data().get('metadata_put_url')
        md = server.metadata_get()
        metadata_url = md['os-collect-config']['request']['metadata_url']
        self.assertNotEqual(metadata_url, metadata_put_url)

        container_name = 'my-custom-container'
        object_name = '0'
        test_path = '/v1/AUTH_test_tenant_id/%s/%s' % (
            container_name, object_name)
        self.assertEqual(test_path, urlparse.urlparse(metadata_put_url).path)
        self.assertEqual(test_path, urlparse.urlparse(metadata_url).path)
        sc.put_object.assert_called_once_with(
            container_name, object_name, jsonutils.dumps(md))

        sc.head_container.return_value = {'x-container-object-count': '0'}
        server._delete_temp_url()
        sc.delete_object.assert_called_once_with(container_name, object_name)
        sc.head_container.assert_called_once_with(container_name)
        sc.delete_container.assert_called_once_with(container_name)
        return metadata_url, server

    def test_server_create_software_config_poll_temp_url(self):
        metadata_url, server = (
            self._server_create_software_config_poll_temp_url())

        self.assertEqual({
            'os-collect-config': {
                'request': {
                    'metadata_url': metadata_url
                },
                'collectors': ['request', 'local']
            },
            'deployments': []
        }, server.metadata_get())

    def _server_create_software_config(self,
                                       server_name='server_sc',
                                       md=None,
                                       ret_tmpl=False):
        stack_name = '%s_s' % server_name
        (tmpl, stack) = self._setup_test_stack(stack_name, server_sc_tmpl)
        self.stack = stack
        self.server_props = tmpl.t['resources']['server']['properties']
        if md is not None:
            tmpl.t['resources']['server']['metadata'] = md

        stack.stack_user_project_id = '8888'
        resource_defns = tmpl.resource_definitions(stack)
        server = deployed_server.DeployedServer(
            'server', resource_defns['server'], stack)
        self.patchobject(server, 'heat')
        scheduler.TaskRunner(server.create)()

        self.assertEqual('4567', server.access_key)
        self.assertEqual('8901', server.secret_key)
        self.assertEqual('1234', server._get_user_id())
        self.assertEqual('POLL_SERVER_CFN',
                         server.properties.get('software_config_transport'))

        self.assertTrue(stack.access_allowed('4567', 'server'))
        self.assertFalse(stack.access_allowed('45678', 'server'))
        self.assertFalse(stack.access_allowed('4567', 'wserver'))
        if ret_tmpl:
            return server, tmpl
        else:
            return server

    @mock.patch.object(heat_plugin.HeatClientPlugin, 'url_for')
    def test_server_create_software_config(self, fake_url):
        fake_url.return_value = 'the-cfn-url'
        server = self._server_create_software_config()

        self.assertEqual({
            'os-collect-config': {
                'cfn': {
                    'access_key_id': '4567',
                    'metadata_url': 'the-cfn-url/v1/',
                    'path': 'server.Metadata',
                    'secret_access_key': '8901',
                    'stack_name': 'server_sc_s'
                },
                'collectors': ['cfn', 'local']
            },
            'deployments': []
        }, server.metadata_get())

    @mock.patch.object(heat_plugin.HeatClientPlugin, 'url_for')
    def test_server_create_software_config_metadata(self, fake_url):
        md = {'os-collect-config': {'polling_interval': 10}}
        fake_url.return_value = 'the-cfn-url'
        server = self._server_create_software_config(md=md)

        self.assertEqual({
            'os-collect-config': {
                'cfn': {
                    'access_key_id': '4567',
                    'metadata_url': 'the-cfn-url/v1/',
                    'path': 'server.Metadata',
                    'secret_access_key': '8901',
                    'stack_name': 'server_sc_s'
                },
                'collectors': ['cfn', 'local'],
                'polling_interval': 10
            },
            'deployments': []
        }, server.metadata_get())

    def _server_create_software_config_poll_heat(self,
                                                 server_name='server_heat',
                                                 md=None):
        stack_name = '%s_s' % server_name
        (tmpl, stack) = self._setup_test_stack(stack_name, server_heat_tmpl)
        self.stack = stack
        props = tmpl.t['resources']['server']['properties']
        props['software_config_transport'] = 'POLL_SERVER_HEAT'
        if md is not None:
            tmpl.t['resources']['server']['metadata'] = md
        self.server_props = props

        resource_defns = tmpl.resource_definitions(stack)
        server = deployed_server.DeployedServer(
            'server', resource_defns['server'], stack)

        scheduler.TaskRunner(server.create)()
        self.assertEqual('1234', server._get_user_id())

        self.assertTrue(stack.access_allowed('1234', 'server'))
        self.assertFalse(stack.access_allowed('45678', 'server'))
        self.assertFalse(stack.access_allowed('4567', 'wserver'))
        return stack, server

    def test_server_software_config_poll_heat(self):
        stack, server = self._server_create_software_config_poll_heat()
        md = {
            'os-collect-config': {
                'heat': {
                    'auth_url': 'http://server.test:5000/v2.0',
                    'password': server.password,
                    'project_id': '8888',
                    'region_name': 'RegionOne',
                    'resource_name': 'server',
                    'stack_id': 'server_heat_s/%s' % stack.id,
                    'user_id': '1234'
                },
                'collectors': ['heat', 'local']
            },
            'deployments': []
        }

        self.assertEqual(md, server.metadata_get())
        # update resource.metadata
        md1 = {'os-collect-config': {'polling_interval': 10}}
        server.stack.t.t['resources']['server']['metadata'] = md1
        resource_defns = server.stack.t.resource_definitions(server.stack)
        scheduler.TaskRunner(server.update, resource_defns['server'])()

        occ = md['os-collect-config']
        occ.update(md1['os-collect-config'])
        # os-collect-config merged
        self.assertEqual(md, server.metadata_get())

    def test_server_create_software_config_poll_heat_metadata(self):
        md = {'os-collect-config': {'polling_interval': 10}}
        stack, server = self._server_create_software_config_poll_heat(md=md)

        self.assertEqual({
            'os-collect-config': {
                'heat': {
                    'auth_url': 'http://server.test:5000/v2.0',
                    'password': server.password,
                    'project_id': '8888',
                    'region_name': 'RegionOne',
                    'resource_name': 'server',
                    'stack_id': 'server_heat_s/%s' % stack.id,
                    'user_id': '1234'
                },
                'collectors': ['heat', 'local'],
                'polling_interval': 10
            },
            'deployments': []
        }, server.metadata_get())

    def _server_create_software_config_zaqar(self,
                                             server_name='server_zaqar',
                                             md=None):
        stack_name = '%s_s' % server_name
        (tmpl, stack) = self._setup_test_stack(stack_name, server_zaqar_tmpl)
        self.stack = stack
        props = tmpl.t['resources']['server']['properties']
        props['software_config_transport'] = 'ZAQAR_MESSAGE'
        if md is not None:
            tmpl.t['resources']['server']['metadata'] = md
        self.server_props = props

        resource_defns = tmpl.resource_definitions(stack)
        server = deployed_server.DeployedServer(
            'server', resource_defns['server'], stack)

        zcc = self.patchobject(zaqar.ZaqarClientPlugin, 'create_for_tenant')
        zc = mock.Mock()
        zcc.return_value = zc
        queue = mock.Mock()
        zc.queue.return_value = queue
        scheduler.TaskRunner(server.create)()

        metadata_queue_id = server.data().get('metadata_queue_id')
        md = server.metadata_get()
        queue_id = md['os-collect-config']['zaqar']['queue_id']
        self.assertEqual(queue_id, metadata_queue_id)

        zc.queue.assert_called_once_with(queue_id)
        queue.post.assert_called_once_with(
            {'body': server.metadata_get(), 'ttl': 3600})

        zc.queue.reset_mock()

        server._delete_queue()

        zc.queue.assert_called_once_with(queue_id)
        zc.queue(queue_id).delete.assert_called_once_with()
        return queue_id, server

    def test_server_create_software_config_zaqar(self):
        queue_id, server = self._server_create_software_config_zaqar()
        self.assertEqual({
            'os-collect-config': {
                'zaqar': {
                    'user_id': '1234',
                    'password': server.password,
                    'auth_url': 'http://server.test:5000/v2.0',
                    'project_id': '8888',
                    'region_name': 'RegionOne',
                    'queue_id': queue_id
                },
                'collectors': ['zaqar', 'local']
            },
            'deployments': []
        }, server.metadata_get())

    def test_server_create_software_config_zaqar_metadata(self):
        md = {'os-collect-config': {'polling_interval': 10}}
        queue_id, server = self._server_create_software_config_zaqar(md=md)
        self.assertEqual({
            'os-collect-config': {
                'zaqar': {
                    'user_id': '1234',
                    'password': server.password,
                    'auth_url': 'http://server.test:5000/v2.0',
                    'project_id': '8888',
                    'region_name': 'RegionOne',
                    'queue_id': queue_id
                },
                'collectors': ['zaqar', 'local'],
                'polling_interval': 10
            },
            'deployments': []
        }, server.metadata_get())

    def test_resolve_attribute_os_collect_config(self):
        metadata_url, server = (
            self._server_create_software_config_poll_temp_url())

        # FnGetAtt usage belows requires the resource to have a stack set
        (tmpl, stack) = self._setup_test_stack('stack_name')
        server.stack = stack

        self.assertEqual({
            'request': {
                'metadata_url': metadata_url
            },
            'collectors': ['request', 'local']
        }, server.FnGetAtt('os_collect_config'))



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from heat.common import exception
from heat.common import template_format
from heat.engine import resource
from heat.engine.resources.openstack.heat import delay
from heat.engine import status
from heat.tests import common
from heat.tests import utils

from oslo_utils import fixture as utils_fixture
from oslo_utils import timeutils


class TestDelay(common.HeatTestCase):

    simple_template = template_format.parse('''
heat_template_version: '2016-10-14'
resources:
  constant:
    type: OS::Heat::Delay
    properties:
      min_wait: 3
  variable:
    type: OS::Heat::Delay
    properties:
      min_wait: 1.6
      max_jitter: 4.2
      actions:
        - CREATE
        - DELETE
  variable_prod:
    type: OS::Heat::Delay
    properties:
      min_wait: 2
      max_jitter: 666
      jitter_multiplier: 0.1
      actions:
        - DELETE
''')

    def test_delay_params(self):
        stk = utils.parse_stack(self.simple_template)

        self.assertEqual((3, 0), stk['constant']._delay_parameters())

        self.assertEqual((1.6, 4.2), stk['variable']._delay_parameters())

        min_wait, max_jitter = stk['variable_prod']._delay_parameters()
        self.assertEqual(2, min_wait)
        self.assertAlmostEqual(66.6, max_jitter)

    def test_wait_secs_create(self):
        stk = utils.parse_stack(self.simple_template)
        action = status.ResourceStatus.CREATE

        self.assertEqual(3, stk['constant']._wait_secs(action))

        variable = stk['variable']._wait_secs(action)
        self.assertGreaterEqual(variable, 1.6)
        self.assertLessEqual(variable, 5.8)
        self.assertNotEqual(variable, stk['variable']._wait_secs(action))

        self.assertEqual(0, stk['variable_prod']._wait_secs(action))

    def test_wait_secs_delete(self):
        stk = utils.parse_stack(self.simple_template)
        action = status.ResourceStatus.DELETE

        self.assertEqual(0, stk['constant']._wait_secs(action))

        variable = stk['variable']._wait_secs(action)
        self.assertGreaterEqual(variable, 1.6)
        self.assertLessEqual(variable, 5.8)
        self.assertNotEqual(variable, stk['variable']._wait_secs(action))

        variable_prod = stk['variable_prod']._wait_secs(action)
        self.assertGreaterEqual(variable_prod, 2.0)
        self.assertLessEqual(variable_prod, 68.6)
        self.assertNotEqual(variable_prod,
                            stk['variable_prod']._wait_secs(action))

    def test_wait_secs_update(self):
        stk = utils.parse_stack(self.simple_template)
        action = status.ResourceStatus.UPDATE

        self.assertEqual(0, stk['constant']._wait_secs(action))
        self.assertEqual(0, stk['variable']._wait_secs(action))
        self.assertEqual(0, stk['variable_prod']._wait_secs(action))

    def test_validate_success(self):
        stk = utils.parse_stack(self.simple_template)
        for res in stk.resources.values():
            self.assertIsNone(res.validate())

    def test_validate_failure(self):
        stk = utils.parse_stack(self.simple_template)
        stk.timeout_mins = 1
        self.assertRaises(exception.StackValidationFailed,
                          stk['variable_prod'].validate)


class DelayCompletionTest(common.HeatTestCase):
    def setUp(self):
        super(DelayCompletionTest, self).setUp()
        self.time_fixture = utils_fixture.TimeFixture()
        self.useFixture(self.time_fixture)

    def test_complete_no_wait(self):
        now = timeutils.utcnow()
        self.time_fixture.advance_time_seconds(-1)
        self.assertEqual(True, delay.Delay._check_complete(now, 0))

    def test_complete(self):
        now = timeutils.utcnow()
        self.time_fixture.advance_time_seconds(5.1)
        self.assertEqual(True, delay.Delay._check_complete(now, 5.1))

    def test_already_complete(self):
        now = timeutils.utcnow()
        self.time_fixture.advance_time_seconds(5.1)
        self.assertEqual(True, delay.Delay._check_complete(now, 5))

    def test_incomplete_short_delay(self):
        now = timeutils.utcnow()
        self.time_fixture.advance_time_seconds(2)
        self.assertEqual(False, delay.Delay._check_complete(now, 5))

    def test_incomplete_moderate_delay(self):
        now = timeutils.utcnow()
        self.time_fixture.advance_time_seconds(2)
        poll_del = self.assertRaises(resource.PollDelay,
                                     delay.Delay._check_complete,
                                     now, 6)
        self.assertEqual(2, poll_del.period)

    def test_incomplete_long_delay(self):
        now = timeutils.utcnow()
        self.time_fixture.advance_time_seconds(0.1)
        poll_del = self.assertRaises(resource.PollDelay,
                                     delay.Delay._check_complete,
                                     now, 62)
        self.assertEqual(30, poll_del.period)



#
# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from unittest import mock

import fixtures

from heat.common import exception
from heat.common.i18n import _
from heat.tests import common


class TestException(exception.HeatException):
    msg_fmt = _("Testing message %(text)s")


class TestHeatException(common.HeatTestCase):

    def test_fatal_exception_error(self):
        self.useFixture(fixtures.MonkeyPatch(
            'heat.common.exception._FATAL_EXCEPTION_FORMAT_ERRORS',
            True))
        self.assertRaises(KeyError, TestException)

    def test_format_string_error_message(self):
        message = "This format %(message)s should work"
        err = exception.Error(message)
        self.assertEqual(message, str(err))


class TestStackValidationFailed(common.HeatTestCase):

    scenarios = [
        ('test_error_as_exception', dict(
            kwargs=dict(
                error=exception.StackValidationFailed(
                    error='Error',
                    path=['some', 'path'],
                    message='Some message')),
            expected='Error: some.path: Some message',
            called_error='Error',
            called_path=['some', 'path'],
            called_msg='Some message'
        )),
        ('test_full_exception', dict(
            kwargs=dict(
                error='Error',
                path=['some', 'path'],
                message='Some message'),
            expected='Error: some.path: Some message',
            called_error='Error',
            called_path=['some', 'path'],
            called_msg='Some message'
        )),
        ('test_no_error_exception', dict(
            kwargs=dict(
                path=['some', 'path'],
                message='Chain letter'),
            expected='some.path: Chain letter',
            called_error='',
            called_path=['some', 'path'],
            called_msg='Chain letter'
        )),
        ('test_no_path_exception', dict(
            kwargs=dict(
                error='Error',
                message='Just no.'),
            expected='Error: Just no.',
            called_error='Error',
            called_path=[],
            called_msg='Just no.'
        )),
        ('test_no_msg_exception', dict(
            kwargs=dict(
                error='Error',
                path=['we', 'lost', 'our', 'message']),
            expected='Error: we.lost.our.message: ',
            called_error='Error',
            called_path=['we', 'lost', 'our', 'message'],
            called_msg=''
        )),
        ('test_old_format_exception', dict(
            kwargs=dict(
                message='Wow. I think I am old error message format.'
            ),
            expected='Wow. I think I am old error message format.',
            called_error='',
            called_path=[],
            called_msg='Wow. I think I am old error message format.'
        )),
        ('test_int_path_item_exception', dict(
            kwargs=dict(
                path=['null', 0]
            ),
            expected='null[0]: ',
            called_error='',
            called_path=['null', 0],
            called_msg=''
        )),
        ('test_digit_path_item_exception', dict(
            kwargs=dict(
                path=['null', '0']
            ),
            expected='null[0]: ',
            called_error='',
            called_path=['null', '0'],
            called_msg=''
        )),
        ('test_string_path_exception', dict(
            kwargs=dict(
                path='null[0].not_null'
            ),
            expected='null[0].not_null: ',
            called_error='',
            called_path=['null[0].not_null'],
            called_msg=''
        ))
    ]

    def test_exception(self):
        try:
            raise exception.StackValidationFailed(**self.kwargs)
        except exception.StackValidationFailed as ex:
            self.assertIn(self.expected, str(ex))
            self.assertIn(self.called_error, ex.error)
            self.assertEqual(self.called_path, ex.path)
            self.assertEqual(self.called_msg, ex.error_message)


class TestResourceFailure(common.HeatTestCase):
    def test_status_reason_resource(self):
        reason = ('Resource CREATE failed: ValueError: resources.oops: '
                  'Test Resource failed oops')

        exc = exception.ResourceFailure(reason, None, action='CREATE')
        self.assertEqual('ValueError', exc.error)
        self.assertEqual(['resources', 'oops'], exc.path)
        self.assertEqual('Test Resource failed oops', exc.error_message)

    def test_status_reason_general(self):
        reason = ('something strange happened')
        exc = exception.ResourceFailure(reason, None, action='CREATE')
        self.assertEqual('', exc.error)
        self.assertEqual([], exc.path)
        self.assertEqual('something strange happened', exc.error_message)

    def test_status_reason_general_res(self):
        res = mock.Mock()
        res.name = 'fred'
        res.stack.t.get_section_name.return_value = 'Resources'

        reason = ('something strange happened')
        exc = exception.ResourceFailure(reason, res, action='CREATE')
        self.assertEqual('', exc.error)
        self.assertEqual(['Resources', 'fred'], exc.path)
        self.assertEqual('something strange happened', exc.error_message)

    def test_std_exception(self):
        base_exc = ValueError('sorry mom')
        exc = exception.ResourceFailure(base_exc, None, action='UPDATE')
        self.assertEqual('ValueError', exc.error)
        self.assertEqual([], exc.path)
        self.assertEqual('sorry mom', exc.error_message)

    def test_std_exception_with_resource(self):
        base_exc = ValueError('sorry mom')
        res = mock.Mock()
        res.name = 'fred'
        res.stack.t.get_section_name.return_value = 'Resources'
        exc = exception.ResourceFailure(base_exc, res, action='UPDATE')
        self.assertEqual('ValueError', exc.error)
        self.assertEqual(['Resources', 'fred'], exc.path)
        self.assertEqual('sorry mom', exc.error_message)

    def test_heat_exception(self):
        base_exc = ValueError('sorry mom')
        heat_exc = exception.ResourceFailure(base_exc, None, action='UPDATE')
        exc = exception.ResourceFailure(heat_exc, None, action='UPDATE')
        self.assertEqual('ValueError', exc.error)
        self.assertEqual([], exc.path)
        self.assertEqual('sorry mom', exc.error_message)

    def test_nested_exceptions(self):
        res = mock.Mock()
        res.name = 'frodo'
        res.stack.t.get_section_name.return_value = 'Resources'

        reason = ('Resource UPDATE failed: ValueError: resources.oops: '
                  'Test Resource failed oops')
        base_exc = exception.ResourceFailure(reason, res, action='UPDATE')
        exc = exception.ResourceFailure(base_exc, res, action='UPDATE')
        self.assertEqual(['Resources', 'frodo', 'resources', 'oops'], exc.path)
        self.assertEqual('ValueError', exc.error)
        self.assertEqual('Test Resource failed oops', exc.error_message)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from heat.common import exception
from heat.common.i18n import _
from heat.engine import constraints
from heat.engine import properties
from heat.engine.resources.openstack.neutron import neutron
from heat.engine import support
from heat.engine import translation


class RBACPolicy(neutron.NeutronResource):
    """A Resource for managing RBAC policy in Neutron.

    This resource creates and manages Neutron RBAC policy,
    which allows to share Neutron networks and qos-policies
    to subsets of tenants.
    """

    support_status = support.SupportStatus(version='6.0.0')

    required_service_extension = 'rbac-policies'

    entity = 'rbac_policy'

    PROPERTIES = (
        OBJECT_TYPE, TARGET_TENANT, ACTION, OBJECT_ID, TENANT_ID
    ) = (
        'object_type', 'target_tenant', 'action', 'object_id', 'tenant_id'
    )

    OBJECT_TYPE_KEYS = (
        OBJECT_NETWORK, OBJECT_QOS_POLICY,
    ) = (
        'network', 'qos_policy',
    )

    ACTION_KEYS = (
        ACCESS_AS_SHARED, ACCESS_AS_EXTERNAL,
    ) = (
        'access_as_shared', 'access_as_external',
    )

    # Change it when neutron supports more function in the future.
    SUPPORTED_TYPES_ACTIONS = {
        OBJECT_NETWORK: [ACCESS_AS_SHARED, ACCESS_AS_EXTERNAL],
        OBJECT_QOS_POLICY: [ACCESS_AS_SHARED],
    }

    properties_schema = {
        OBJECT_TYPE: properties.Schema(
            properties.Schema.STRING,
            _('Type of the object that RBAC policy affects.'),
            required=True,
            constraints=[
                constraints.AllowedValues(OBJECT_TYPE_KEYS)
            ]
        ),
        TARGET_TENANT: properties.Schema(
            properties.Schema.STRING,
            _('ID of the tenant to which the RBAC policy will be enforced.'),
            required=True,
            update_allowed=True
        ),
        ACTION: properties.Schema(
            properties.Schema.STRING,
            _('Action for the RBAC policy. The allowed actions differ for '
              'different object types - only %(network)s objects can have an '
              '%(external)s action.') % {'network': OBJECT_NETWORK,
                                         'external': ACCESS_AS_EXTERNAL},
            required=True,
            constraints=[
                constraints.AllowedValues(ACTION_KEYS)
            ]
        ),
        OBJECT_ID: properties.Schema(
            properties.Schema.STRING,
            _('ID or name of the RBAC object.'),
            required=True
        ),
        TENANT_ID: properties.Schema(
            properties.Schema.STRING,
            _('The owner tenant ID. Only required if the caller has an '
              'administrative role and wants to create a RBAC for another '
              'tenant.')
        )
    }

    def translation_rules(self, props):
        return [
            translation.TranslationRule(
                props,
                translation.TranslationRule.RESOLVE,
                [self.OBJECT_ID],
                client_plugin=self.client_plugin(),
                finder='find_resourceid_by_name_or_id',
                entity=self._get_client_res_type(props[self.OBJECT_TYPE])
            )
        ]

    def _get_client_res_type(self, object_type):
        client_plugin = self.client_plugin()
        if object_type == self.OBJECT_NETWORK:
            return client_plugin.RES_TYPE_NETWORK
        elif object_type == self.OBJECT_QOS_POLICY:
            return client_plugin.RES_TYPE_QOS_POLICY
        else:
            return object_type

    def handle_create(self):
        props = self.prepare_properties(
            self.properties,
            self.physical_resource_name())
        rbac = self.client().create_rbac_policy(
            {'rbac_policy': props})['rbac_policy']
        self.resource_id_set(rbac['id'])

    def handle_update(self, json_snippet, tmpl_diff, prop_diff):
        if prop_diff:
            self.client().update_rbac_policy(
                self.resource_id, {'rbac_policy': prop_diff})

    def handle_delete(self):
        if self.resource_id is not None:
            with self.client_plugin().ignore_not_found:
                self.client().delete_rbac_policy(self.resource_id)

    def validate(self):
        """Validate the provided properties."""
        super(RBACPolicy, self).validate()

        action = self.properties[self.ACTION]
        obj_type = self.properties[self.OBJECT_TYPE]

        # Validate obj_type and action per SUPPORTED_TYPES_ACTIONS.
        if action not in self.SUPPORTED_TYPES_ACTIONS[obj_type]:
            valid_actions = ', '.join(self.SUPPORTED_TYPES_ACTIONS[obj_type])
            msg = (_('Invalid action "%(action)s" for object type '
                     '%(obj_type)s. Valid actions: %(valid_actions)s') %
                   {'action': action, 'obj_type': obj_type,
                    'valid_actions': valid_actions})
            properties_section = self.properties.error_prefix[0]
            path = [self.stack.t.RESOURCES, self.t.name,
                    self.stack.t.get_section_name(properties_section),
                    self.ACTION]
            raise exception.StackValidationFailed(error='Property error',
                                                  path=path,
                                                  message=msg)


def resource_mapping():
    return {
        'OS::Neutron::RBACPolicy': RBACPolicy,
    }



#    Copyright 2015 Intel Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Heat common internal object model"""

import weakref

from oslo_versionedobjects import base as ovoo_base


class HeatObjectRegistry(ovoo_base.VersionedObjectRegistry):
    pass


class HeatObject(ovoo_base.VersionedObject):
    OBJ_PROJECT_NAMESPACE = 'heat'
    VERSION = '1.0'

    @property
    def _context(self):
        if self._contextref is None:
            return
        ctxt = self._contextref()
        assert ctxt is not None, "Need a reference to the context"
        return ctxt

    @_context.setter
    def _context(self, context):
        if context:
            self._contextref = weakref.ref(context)
        else:
            self._contextref = None



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_policy import policy

from heat.policies import base

POLICY_ROOT = 'resource_types:%s'

resource_types_policies = [
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Nova::Flavor',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Cinder::EncryptedVolumeType',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Cinder::VolumeType',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Cinder::Quota',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Neutron::Quota',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Nova::Quota',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Octavia::Quota',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Manila::ShareType',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Neutron::ProviderNet',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Neutron::QoSPolicy',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Neutron::QoSBandwidthLimitRule',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Neutron::QoSDscpMarkingRule',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Neutron::QoSMinimumBandwidthRule',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Neutron::QoSMinimumPacketRateRule',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Neutron::Segment',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Nova::HostAggregate',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Cinder::QoSSpecs',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Cinder::QoSAssociation',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Keystone::*',
        check_str=base.RULE_PROJECT_ADMIN,
        # NOTE(tkajinam): Keystone does not yet support project scope SRBAC
        #                 but we limit the scope to project one here to make
        #                 this consistent with the scope enforcement in stack
        #                 resources.
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Blazar::Host',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Octavia::Flavor',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Octavia::FlavorProfile',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Octavia::AvailabilityZone',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project']),
    policy.RuleDefault(
        name=POLICY_ROOT % 'OS::Octavia::AvailabilityZoneProfile',
        check_str=base.RULE_PROJECT_ADMIN,
        scope_types=['project'])
]


def list_rules():
    return resource_types_policies



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from neutronclient.common import exceptions
from neutronclient.v2_0 import client as neutronclient

from heat.common import exception
from heat.common import template_format
from heat.engine.resources.openstack.neutron import metering
from heat.engine import scheduler
from heat.tests import common
from heat.tests import utils


metering_template = '''
heat_template_version: 2015-04-30
description: Template to test metering resources
resources:
  label:
    type: OS::Neutron::MeteringLabel
    properties:
      name: TestLabel
      description: Description of TestLabel
      shared: True

  rule:
    type: OS::Neutron::MeteringRule
    properties:
      metering_label_id: { get_resource: label }
      remote_ip_prefix: 10.0.3.0/24
      direction: ingress
      excluded: false
'''


class MeteringLabelTest(common.HeatTestCase):

    def setUp(self):
        super(MeteringLabelTest, self).setUp()
        self.mockclient = mock.Mock(spec=neutronclient.Client)
        self.patchobject(neutronclient, 'Client', return_value=self.mockclient)

    def create_metering_label(self):
        self.mockclient.create_metering_label.return_value = {
            'metering_label': {'id': '1234'}
        }
        snippet = template_format.parse(metering_template)
        self.stack = utils.parse_stack(snippet)
        resource_defns = self.stack.t.resource_definitions(self.stack)
        return metering.MeteringLabel(
            'label', resource_defns['label'], self.stack)

    def test_create(self):
        rsrc = self.create_metering_label()

        scheduler.TaskRunner(rsrc.create)()
        self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)

        self.mockclient.create_metering_label.assert_called_once_with({
            'metering_label': {
                'name': 'TestLabel',
                'description': 'Description of TestLabel',
                'shared': True
            }
        })

    def test_create_failed(self):
        self.mockclient.create_metering_label.side_effect = (
            exceptions.NeutronClientException())

        snippet = template_format.parse(metering_template)
        stack = utils.parse_stack(snippet)
        resource_defns = stack.t.resource_definitions(stack)
        rsrc = metering.MeteringLabel(
            'label', resource_defns['label'], stack)
        error = self.assertRaises(exception.ResourceFailure,
                                  scheduler.TaskRunner(rsrc.create))
        self.assertEqual(
            'NeutronClientException: resources.label: '
            'An unknown exception occurred.',
            str(error))
        self.assertEqual((rsrc.CREATE, rsrc.FAILED), rsrc.state)

        self.mockclient.create_metering_label.assert_called_once_with({
            'metering_label': {
                'name': 'TestLabel',
                'description': 'Description of TestLabel',
                'shared': True
            }
        })

    def test_delete(self):
        rsrc = self.create_metering_label()
        self.mockclient.delete_metering_label.return_value = None
        self.mockclient.show_metering_label.side_effect = (
            exceptions.NeutronClientException(status_code=404))

        scheduler.TaskRunner(rsrc.create)()
        scheduler.TaskRunner(rsrc.delete)()
        self.assertEqual((rsrc.DELETE, rsrc.COMPLETE), rsrc.state)

        self.mockclient.create_metering_label.assert_called_once_with({
            'metering_label': {
                'name': 'TestLabel',
                'description': 'Description of TestLabel',
                'shared': True
            }
        })
        self.mockclient.delete_metering_label.assert_called_once_with('1234')
        self.mockclient.show_metering_label.assert_called_once_with('1234')

    def test_delete_already_gone(self):
        rsrc = self.create_metering_label()
        self.mockclient.delete_metering_label.side_effect = (
            exceptions.NeutronClientException(status_code=404))

        scheduler.TaskRunner(rsrc.create)()
        scheduler.TaskRunner(rsrc.delete)()
        self.assertEqual((rsrc.DELETE, rsrc.COMPLETE), rsrc.state)

        self.mockclient.create_metering_label.assert_called_once_with({
            'metering_label': {
                'name': 'TestLabel',
                'description': 'Description of TestLabel',
                'shared': True
            }
        })
        self.mockclient.delete_metering_label.assert_called_once_with('1234')
        self.mockclient.show_metering_label.assert_not_called()

    def test_delete_failed(self):
        rsrc = self.create_metering_label()
        self.mockclient.delete_metering_label.side_effect = (
            exceptions.NeutronClientException(status_code=400))

        scheduler.TaskRunner(rsrc.create)()
        error = self.assertRaises(exception.ResourceFailure,
                                  scheduler.TaskRunner(rsrc.delete))
        self.assertEqual(
            'NeutronClientException: resources.label: '
            'An unknown exception occurred.',
            str(error))
        self.assertEqual((rsrc.DELETE, rsrc.FAILED), rsrc.state)

        self.mockclient.create_metering_label.assert_called_once_with({
            'metering_label': {
                'name': 'TestLabel',
                'description': 'Description of TestLabel',
                'shared': True
            }
        })
        self.mockclient.delete_metering_label.assert_called_once_with('1234')

    def test_attribute(self):
        rsrc = self.create_metering_label()
        self.mockclient.show_metering_label.return_value = {
            'metering_label': {
                'name': 'TestLabel',
                'description': 'Description of TestLabel',
                'shared': True
            }
        }

        scheduler.TaskRunner(rsrc.create)()
        self.assertEqual('TestLabel', rsrc.FnGetAtt('name'))
        self.assertEqual('Description of TestLabel',
                         rsrc.FnGetAtt('description'))
        self.assertTrue(rsrc.FnGetAtt('shared'))

        self.mockclient.create_metering_label.assert_called_once_with({
            'metering_label': {
                'name': 'TestLabel',
                'description': 'Description of TestLabel',
                'shared': True
            }
        })
        self.mockclient.show_metering_label.assert_called_with('1234')


class MeteringRuleTest(common.HeatTestCase):

    def setUp(self):
        super(MeteringRuleTest, self).setUp()
        self.mockclient = mock.Mock(spec=neutronclient.Client)
        self.patchobject(neutronclient, 'Client', return_value=self.mockclient)

    def create_metering_label_rule(self):
        self.mockclient.create_metering_label_rule.return_value = {
            'metering_label_rule': {'id': '5678'}
        }

        snippet = template_format.parse(metering_template)
        self.stack = utils.parse_stack(snippet)
        self.patchobject(self.stack['label'], 'FnGetRefId',
                         return_value='1234')
        resource_defns = self.stack.t.resource_definitions(self.stack)
        return metering.MeteringRule(
            'rule', resource_defns['rule'], self.stack)

    def test_create(self):
        rsrc = self.create_metering_label_rule()

        scheduler.TaskRunner(rsrc.create)()
        self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)

        self.mockclient.create_metering_label_rule.assert_called_once_with({
            'metering_label_rule': {
                'metering_label_id': '1234',
                'remote_ip_prefix': '10.0.3.0/24',
                'direction': 'ingress',
                'excluded': False
            }
        })

    def test_create_failed(self):
        self.mockclient.create_metering_label_rule.side_effect = (
            exceptions.NeutronClientException())

        snippet = template_format.parse(metering_template)
        stack = utils.parse_stack(snippet)
        self.patchobject(stack['label'], 'FnGetRefId',
                         return_value='1234')
        resource_defns = stack.t.resource_definitions(stack)
        rsrc = metering.MeteringRule(
            'rule', resource_defns['rule'], stack)
        error = self.assertRaises(exception.ResourceFailure,
                                  scheduler.TaskRunner(rsrc.create))
        self.assertEqual(
            'NeutronClientException: resources.rule: '
            'An unknown exception occurred.',
            str(error))
        self.assertEqual((rsrc.CREATE, rsrc.FAILED), rsrc.state)

        self.mockclient.create_metering_label_rule.assert_called_once_with({
            'metering_label_rule': {
                'metering_label_id': '1234',
                'remote_ip_prefix': '10.0.3.0/24',
                'direction': 'ingress',
                'excluded': False
            }
        })

    def test_delete(self):
        rsrc = self.create_metering_label_rule()
        self.mockclient.delete_metering_label_rule.return_value = None
        self.mockclient.show_metering_label_rule.side_effect = (
            exceptions.NeutronClientException(status_code=404))

        scheduler.TaskRunner(rsrc.create)()
        scheduler.TaskRunner(rsrc.delete)()
        self.assertEqual((rsrc.DELETE, rsrc.COMPLETE), rsrc.state)

        self.mockclient.create_metering_label_rule.assert_called_once_with({
            'metering_label_rule': {
                'metering_label_id': '1234',
                'remote_ip_prefix': '10.0.3.0/24',
                'direction': 'ingress',
                'excluded': False
            }
        })
        self.mockclient.delete_metering_label_rule.assert_called_once_with(
            '5678')
        self.mockclient.show_metering_label_rule.assert_called_once_with(
            '5678')

    def test_delete_already_gone(self):
        rsrc = self.create_metering_label_rule()
        self.mockclient.delete_metering_label_rule.side_effect = (
            exceptions.NeutronClientException(status_code=404))

        scheduler.TaskRunner(rsrc.create)()
        scheduler.TaskRunner(rsrc.delete)()
        self.assertEqual((rsrc.DELETE, rsrc.COMPLETE), rsrc.state)

        self.mockclient.create_metering_label_rule.assert_called_once_with({
            'metering_label_rule': {
                'metering_label_id': '1234',
                'remote_ip_prefix': '10.0.3.0/24',
                'direction': 'ingress',
                'excluded': False
            }
        })
        self.mockclient.delete_metering_label_rule.assert_called_once_with(
            '5678')
        self.mockclient.show_metering_label_rule.assert_not_called()

    def test_delete_failed(self):
        rsrc = self.create_metering_label_rule()
        self.mockclient.delete_metering_label_rule.side_effect = (
            exceptions.NeutronClientException(status_code=400))

        scheduler.TaskRunner(rsrc.create)()
        error = self.assertRaises(exception.ResourceFailure,
                                  scheduler.TaskRunner(rsrc.delete))
        self.assertEqual(
            'NeutronClientException: resources.rule: '
            'An unknown exception occurred.',
            str(error))
        self.assertEqual((rsrc.DELETE, rsrc.FAILED), rsrc.state)

        self.mockclient.create_metering_label_rule.assert_called_once_with({
            'metering_label_rule': {
                'metering_label_id': '1234',
                'remote_ip_prefix': '10.0.3.0/24',
                'direction': 'ingress',
                'excluded': False
            }
        })
        self.mockclient.delete_metering_label_rule.assert_called_once_with(
            '5678')
        self.mockclient.show_metering_label_rule.assert_not_called()

    def test_attribute(self):
        rsrc = self.create_metering_label_rule()
        self.mockclient.show_metering_label_rule.return_value = {
            'metering_label_rule': {
                'metering_label_id': '1234',
                'remote_ip_prefix': '10.0.3.0/24',
                'direction': 'ingress',
                'excluded': False
            }
        }

        scheduler.TaskRunner(rsrc.create)()
        self.assertEqual('10.0.3.0/24', rsrc.FnGetAtt('remote_ip_prefix'))
        self.assertEqual('ingress', rsrc.FnGetAtt('direction'))
        self.assertIs(False, rsrc.FnGetAtt('excluded'))

        self.mockclient.create_metering_label_rule.assert_called_once_with({
            'metering_label_rule': {
                'metering_label_id': '1234',
                'remote_ip_prefix': '10.0.3.0/24',
                'direction': 'ingress',
                'excluded': False
            }
        })
        self.mockclient.show_metering_label_rule.assert_called_with('5678')



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from heat.common.i18n import _
from heat.engine import constraints
from heat.engine import properties
from heat.engine.resources.openstack.neutron import neutron
from heat.engine import support
from heat.engine import translation


class PortPair(neutron.NeutronResource):
    """A resource for neutron networking-sfc port-pair.

    This plug-in requires networking-sfc>=1.0.0. So to enable this
    plug-in, install this library and restart the heat-engine.

    A Port Pair represents a service function instance. The ingress port and
    the egress port of the service function may be specified. If a service
    function has one bidirectional port, the ingress port has the same value
    as the egress port.
    """

    support_status = support.SupportStatus(
        version='7.0.0',
        status=support.UNSUPPORTED)

    PROPERTIES = (
        NAME, DESCRIPTION, INGRESS, EGRESS,
        SERVICE_FUNCTION_PARAMETERS,
        ) = (
        'name', 'description', 'ingress', 'egress',
        'service_function_parameters',
    )

    properties_schema = {
        NAME: properties.Schema(
            properties.Schema.STRING,
            _('Name for the Port Pair.'),
            update_allowed=True
        ),
        DESCRIPTION: properties.Schema(
            properties.Schema.STRING,
            _('Description for the Port Pair.'),
            update_allowed=True
        ),
        INGRESS: properties.Schema(
            properties.Schema.STRING,
            _('ID or name of the ingress neutron port.'),
            constraints=[constraints.CustomConstraint('neutron.port')],
            required=True,
        ),
        EGRESS: properties.Schema(
            properties.Schema.STRING,
            _('ID or name of the egress neutron port.'),
            constraints=[constraints.CustomConstraint('neutron.port')],
            required=True,
        ),
        SERVICE_FUNCTION_PARAMETERS: properties.Schema(
            properties.Schema.MAP,
            _('Dictionary of service function parameter. Currently '
              'only correlation=None is supported.'),
            default={'correlation': None},
        ),
    }

    def translation_rules(self, props):
        client_plugin = self.client_plugin()
        return [
            translation.TranslationRule(
                props,
                translation.TranslationRule.RESOLVE,
                [self.INGRESS],
                client_plugin=client_plugin,
                finder='find_resourceid_by_name_or_id',
                entity=client_plugin.RES_TYPE_PORT
            ),
            translation.TranslationRule(
                props,
                translation.TranslationRule.RESOLVE,
                [self.EGRESS],
                client_plugin=client_plugin,
                finder='find_resourceid_by_name_or_id',
                entity=client_plugin.RES_TYPE_PORT
            )
        ]

    def _show_resource(self):
        return self.client_plugin().show_ext_resource('port_pair',
                                                      self.resource_id)

    def handle_create(self):
        props = self.prepare_properties(self.properties,
                                        self.physical_resource_name())
        props['ingress'] = props.get(self.INGRESS)
        props['egress'] = props.get(self.EGRESS)
        port_pair = self.client_plugin().create_ext_resource('port_pair',
                                                             props)
        self.resource_id_set(port_pair['id'])

    def handle_update(self, json_snippet, tmpl_diff, prop_diff):
        if prop_diff:
            self.prepare_update_properties(prop_diff)
            self.client_plugin().update_ext_resource('port_pair', prop_diff,
                                                     self.resource_id)

    def handle_delete(self):
        if self.resource_id is None:
            return
        with self.client_plugin().ignore_not_found:
            self.client_plugin().delete_ext_resource('port_pair',
                                                     self.resource_id)


def resource_mapping():
    return {
        'OS::Neutron::PortPair': PortPair,
    }



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from neutronclient.neutron import v2_0 as neutronV20
from openstack import exceptions
from oslo_utils import excutils

from heat.common import exception
from heat.common import template_format
from heat.engine.clients.os import neutron
from heat.engine import rsrc_defn
from heat.engine import stack
from heat.engine import template
from heat.tests import common
from heat.tests.openstack.neutron import inline_templates
from heat.tests import utils


class NeutronSegmentTest(common.HeatTestCase):

    def setUp(self):
        super(NeutronSegmentTest, self).setUp()

        self.ctx = utils.dummy_context()
        tpl = template_format.parse(inline_templates.SEGMENT_TEMPLATE)
        self.stack = stack.Stack(
            self.ctx,
            'segment_test',
            template.Template(tpl)
        )

        class FakeOpenStackPlugin(object):

            @excutils.exception_filter
            def ignore_not_found(self, ex):
                if not isinstance(ex, exceptions.ResourceNotFound):
                    raise ex

        self.sdkclient = mock.Mock()
        self.patchobject(neutron.NeutronClientPlugin, 'has_extension',
                         return_value=True)
        self.patchobject(excutils.exception_filter, '__exit__')
        self.segment = self.stack['segment']
        self.segment.client = mock.Mock(return_value=self.sdkclient)
        self.segment.client_plugin = mock.Mock(
            return_value=FakeOpenStackPlugin())
        self.patchobject(self.segment, 'physical_resource_name',
                         return_value='test_segment')
        self.patchobject(neutronV20, 'find_resourceid_by_name_or_id',
                         return_value='private')

    def test_segment_handle_create(self):
        seg = mock.Mock(id='9c1eb3fe-7bba-479d-bd43-1d497e53c384')
        create_props = {'name': 'test_segment',
                        'network_id': 'private',
                        'network_type': 'vxlan',
                        'segmentation_id': 101}

        mock_create = self.patchobject(self.sdkclient.network,
                                       'create_segment',
                                       return_value=seg)
        self.segment.handle_create()
        self.assertEqual('9c1eb3fe-7bba-479d-bd43-1d497e53c384',
                         self.segment.resource_id)
        mock_create.assert_called_once_with(**create_props)

    def test_segment_handle_delete(self):
        segment_id = '477e8273-60a7-4c41-b683-fdb0bc7cd151'
        self.segment.resource_id = segment_id
        mock_delete = self.patchobject(self.sdkclient.network,
                                       'delete_segment',
                                       return_value=None)
        self.assertIsNone(self.segment.handle_delete())
        mock_delete.assert_called_once_with(self.segment.resource_id)

    def test_segment_handle_delete_not_found(self):
        segment_id = '477e8273-60a7-4c41-b683-fdb0bc7cd151'
        self.segment.resource_id = segment_id
        mock_delete = self.patchobject(
            self.sdkclient.network, 'delete_segment',
            side_effect=exceptions.ResourceNotFound)
        self.assertIsNone(self.segment.handle_delete())
        mock_delete.assert_called_once_with(self.segment.resource_id)

    def test_segment_delete_resource_id_is_none(self):
        self.segment.resource_id = None
        mock_delete = self.patchobject(self.sdkclient.network,
                                       'delete_segment')
        self.assertIsNone(self.segment.handle_delete())
        self.assertEqual(0, mock_delete.call_count)

    def test_segment_handle_update(self):
        segment_id = '477e8273-60a7-4c41-b683-fdb0bc7cd151'
        self.segment.resource_id = segment_id

        props = {
            'name': 'test_segment',
            'description': 'updated'
        }
        mock_update = self.patchobject(self.sdkclient.network,
                                       'update_segment')
        update_dict = props.copy()
        update_snippet = rsrc_defn.ResourceDefinition(
            self.segment.name,
            self.segment.type(),
            props)

        # with name
        self.segment.handle_update(
            json_snippet=update_snippet,
            tmpl_diff={},
            prop_diff=props)

        # without name
        props['name'] = None
        self.segment.handle_update(
            json_snippet=update_snippet,
            tmpl_diff={},
            prop_diff=props)
        self.assertEqual(2, mock_update.call_count)
        mock_update.assert_called_with(segment_id, **update_dict)

    def test_validate_vlan_type(self):
        self.t = template_format.parse(inline_templates.SEGMENT_TEMPLATE)
        props = self.t['resources']['segment']['properties']
        props['network_type'] = 'vlan'
        self.stack = utils.parse_stack(self.t)
        rsrc = self.stack['segment']
        errMsg = 'physical_network is required for vlan provider network.'
        error = self.assertRaises(exception.StackValidationFailed,
                                  rsrc.validate)
        self.assertEqual(errMsg, str(error))

        props['physical_network'] = 'physnet'
        props['segmentation_id'] = '4095'
        self.stack = utils.parse_stack(self.t)
        rsrc = self.stack['segment']
        errMsg = ('Up to 4094 VLAN network segments can exist '
                  'on each physical_network.')
        error = self.assertRaises(exception.StackValidationFailed,
                                  rsrc.validate)
        self.assertEqual(errMsg, str(error))

    def test_validate_flat_type(self):
        self.t = template_format.parse(inline_templates.SEGMENT_TEMPLATE)
        props = self.t['resources']['segment']['properties']
        props['network_type'] = 'flat'
        props['physical_network'] = 'physnet'
        self.stack = utils.parse_stack(self.t)
        rsrc = self.stack['segment']
        errMsg = ('segmentation_id is prohibited for flat provider network.')
        error = self.assertRaises(exception.StackValidationFailed,
                                  rsrc.validate)
        self.assertEqual(errMsg, str(error))

    def test_validate_tunnel_type(self):
        self.t = template_format.parse(inline_templates.SEGMENT_TEMPLATE)
        props = self.t['resources']['segment']['properties']
        props['network_type'] = 'vxlan'
        props['physical_network'] = 'physnet'
        self.stack = utils.parse_stack(self.t)
        rsrc = self.stack['segment']
        errMsg = ('physical_network is prohibited for vxlan provider network.')
        error = self.assertRaises(exception.StackValidationFailed,
                                  rsrc.validate)
        self.assertEqual(errMsg, str(error))

    def test_segment_get_attr(self):
        segment_id = '477e8273-60a7-4c41-b683-fdb0bc7cd151'
        self.segment.resource_id = segment_id

        seg = {'name': 'test_segment',
               'id': '477e8273-60a7-4c41-b683-fdb0bc7cd151',
               'network_type': 'vxlan',
               'network_id': 'private',
               'segmentation_id': 101}

        class FakeSegment(object):
            def to_dict(self):
                return seg

        get_mock = self.patchobject(self.sdkclient.network, 'get_segment',
                                    return_value=FakeSegment())
        self.assertEqual(seg,
                         self.segment.FnGetAtt('show'))
        get_mock.assert_called_once_with(self.segment.resource_id)

    def test_needs_replace_failed(self):
        self.stack.store()
        self.segment.state_set(self.segment.CREATE, self.segment.FAILED)
        side_effect = [exceptions.ResourceNotFound, 'attr']
        mock_show_resource = self.patchobject(self.segment, '_show_resource',
                                              side_effect=side_effect)
        self.segment.resource_id = None
        self.assertTrue(self.segment.needs_replace_failed())
        self.assertEqual(0, mock_show_resource.call_count)

        self.segment.resource_id = 'seg_id'
        self.assertTrue(self.segment.needs_replace_failed())
        self.assertEqual(1, mock_show_resource.call_count)

        self.assertFalse(self.segment.needs_replace_failed())
        self.assertEqual(2, mock_show_resource.call_count)
